#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os


ROTARYENCODER_PLUGIN_ICON = os.path.join( os.path.dirname(__file__), 'resources', 'rotary.png' )


ROTARYENCODER_PLUGIN_WINDOW_SIZE = 300, 500

