import os

EMULATOR_PLUGIN_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'develop_board.svg')
EMULATOR_LAUNCH_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'launch.svg')
EMULATOR_CHECKED_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'checkbox_checked.svg')
EMULATOR_UNCHECKED_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'checkbox_unchecked.svg')

EMULATOR_PLUGIN_WINDOW_SIZE = 300, 400
