SUBJECTS = '/subjects'
USERS = '/users'
AUTH_TOKEN = '/auth-token'
HEADERS = {
    'Authorization': 'Token 4aab2b946ee6b55ab94162584e5bec702a481e1f',
    'Accept': 'application/json',
    'Content-Type': 'application/json',
}