#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os

SETTINGS_PRIORITY = 80


TIMELINE_PLUGIN_ICON = os.path.join(
    os.path.dirname(__file__), 'resources', 'graph.png')

TIMELINE_PLUGIN_WINDOW_SIZE = 650, 200
TIMELINE_PLUGIN_REFRESH_RATE = 100
